<?php
/* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */

/**
 * WordPress XML-RPC API用基底クラス
 * 
 * WordPress XML-RPC APIにて共通的に使用する機能を集めた用基底クラス
 * 
 * PHP version 5
 * 
 * LICENSE: This LICENSE is in the New BSD license style.
 *
 * @category  WordPress
 * @package   WordPress_Core
 * @author    Takahiro Ooishi <taka0125@users.sourceforge.jp>
 * @version   SVN : $Id$
 * @copyright 2009 Takahiro Ooishi
 * @license   http://opensource.org/licenses/bsd-license.php New BSD License
 * @link      http://code.google.com/p/wpalforphp/wiki/WordPressCoreAPI
 * @see       http://codex.wordpress.org/XML-RPC_wp
 */

/**
 * WordPress XML-RPC API用基底クラス
 * 
 * WordPress XML-RPC APIにて共通的に使用する機能を集めた用基底クラス
 * 
 * @category  WordPress
 * @package   WordPress_Core
 * @abstract
 * @author    Takahiro Ooishi <taka0125@users.sourceforge.jp>
 * @version   Release: @package_version@
 * @copyright 2009 Takahiro Ooishi
 * @license   http://opensource.org/licenses/bsd-license.php New BSD License
 * @link      http://code.google.com/p/wpalforphp/wiki/WordPressCoreAPI
 *
 */
abstract class WordPress_Core_API
{
    protected $config;
    protected $errorFlg;
    protected $faultCode;
    protected $faultString;
    protected $connection;

    const DefaultTimeOut = 10;

    function __construct($config)
    {
        if (!$this->isValidConfig($config)) throw new Exception('invalid config');
        $this->setConfig($config);
        $this->connection = $this->open();
        $this->setTimeout(self::DefaultTimeOut);
    }

    function __destruct()
    {
        if (!is_null($this->connection)) {
            $this->close($this->connection);
            $this->connection = null;
        }
    }

    function config()
    {
        return $this->config;
    }

    function setConfig($config)
    {
        $this->config = $config;
    }

    function setTimeout($timeout)
    {
        if (!is_numeric($timeout)) return false;
        $timeout = intval($timeout);
        if ($timeout < 1) return false;
        
        curl_setopt($this->connection, CURLOPT_TIMEOUT, $timeout);
        return true;
    }

    function isError()
    {
        return $this->errorFlg;
    }

    function getError()
    {
        return "faultCode={$this->faultCode},faultString={$this->faultString}";
    }

    function faultCode()
    {
        return $this->faultCode;
    }

    function faultString()
    {
        return $this->faultString;
    }

    protected function clearError()
    {
        $this->errorFlg = false;
        $this->faultCode = null; 
        $this->faultString = null;
    }

    protected function setError($results)
    {
        if ($this->config->outputEncode() !== WordPress_Encode::Utf8) {
            $results = WordPress_Encode::encode($results, $this->config->outputEncode(), WordPress_Encode::Utf8);
        }

        $this->errorFlg = true;
        $this->faultCode = $results['faultCode'];
        $this->faultString = $results['faultString'];
    }

    protected function open()
    {
        $header = array();
        $header[] = 'Content-type: text/xml';
        
        $connection = curl_init();
        
        $url = $this->config->xmlrpcUrl();
        
        curl_setopt($connection, CURLOPT_URL, $url);
        curl_setopt($connection, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($connection, CURLOPT_TIMEOUT, 1);
        curl_setopt($connection, CURLOPT_HTTPHEADER, $header);
        
        return $connection;
    }
    
    protected function close($connection)
    {
        curl_close($connection);
    }

    protected function buildRequest($function_name, $params)
    {
        $output_options = array(
            'escaping' => array(''),
            'encoding' => 'UTF-8'
        );
        return xmlrpc_encode_request($function_name, $params, $output_options);
    }

    protected function request($request)
    {
        $this->clearError();

        if (is_null($this->connection)) {
            $this->connection = $this->open();
        }

        curl_setopt($this->connection, CURLOPT_POSTFIELDS, $request);

        $xml = curl_exec($this->connection);
        if (curl_errno($this->connection)) throw new Exception(curl_error($this->connection));

        $results = xmlrpc_decode($xml);
        if (!is_array($results)) return $results;

        if (xmlrpc_is_fault($results)) {
            $this->setError($results);
            return false;
        }

        if ($this->config->outputEncode() !== WordPress_Encode::Utf8) {
            $results = WordPress_Encode::encode($results, $this->config->outputEncode(), WordPress_Encode::Utf8);
        }

        return $results;
    }

    protected function isValidConfig($config)
    {
        if (! $config instanceof WordPress_Config) return false;
        return true;
    }
}
